#ifndef _CAN_SYSDEP_H
#define _CAN_SYSDEP_H

#ifdef CAN_WITH_RTL
#include <rtl.h>
#include <rtl_sync.h>
#include <rtl_core.h>
#include <rtl_mutex.h>
#include <rtl_sched.h>
#endif /*CAN_WITH_RTL*/

/*#define __NO_VERSION__*/
/*#include <linux/module.h>*/

#include <linux/version.h>
#include <linux/wait.h>
#include <linux/list.h>
#include <linux/fs.h>
#include <linux/ioport.h>
#include <linux/delay.h>
#include <linux/sched.h>
#include <linux/interrupt.h>
#include <asm/errno.h>

#include <asm/io.h>
#include <asm/atomic.h>
#include <asm/irq.h>
#include <asm/uaccess.h>

#if (LINUX_VERSION_CODE < KERNEL_VERSION(2,4,0))
#include <linux/malloc.h>
#else
#include <linux/slab.h>
#endif


/* Next is not sctrictly correct, because of 2.3.0, 2.3.1, 2.3.2
   kernels need next definitions  too */
#if (LINUX_VERSION_CODE < KERNEL_VERSION(2,2,19)) /* may need correction */
  #define wait_queue_head_t struct wait_queue *
  #define wait_queue_t      struct wait_queue
  #define init_waitqueue_head(queue_head) (*queue_head=NULL)
  #define init_waitqueue_entry(qentry,qtask) \
                        (qentry->next=NULL,qentry->task=qtask)
  #define DECLARE_WAIT_QUEUE_HEAD(name) \
        struct wait_queue * name=NULL
  #define DECLARE_WAITQUEUE(wait, current) \
        struct wait_queue wait = { current, NULL }
  #define init_MUTEX(sem) (*sem=MUTEX)
  #define DECLARE_MUTEX(name) struct semaphore name=MUTEX
#endif /* 2.2.19 */


#if (LINUX_VERSION_CODE < KERNEL_VERSION(2,5,7)) || (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0))

#define MINOR_NR \
	(MINOR(file->f_dentry->d_inode->i_rdev))

#else /* Linux kernel < 2.5.7 or >= 2.6.0 */

#define MINOR_NR \
	(minor(file->f_dentry->d_inode->i_rdev))

#endif /* Linux kernel < 2.5.7 or >= 2.6.0 */

#ifndef CAN_WITH_RTL
#if ((LINUX_VERSION_CODE < KERNEL_VERSION(2,5,68)) && !defined(IRQ_RETVAL))
  typedef void can_irqreturn_t;
  #define CAN_IRQ_NONE
  #define CAN_IRQ_HANDLED
  #define CAN_IRQ_RETVAL(x)
#else /* <=2.5.67 */
  typedef irqreturn_t can_irqreturn_t;
  #define CAN_IRQ_NONE    IRQ_NONE
  #define CAN_IRQ_HANDLED IRQ_HANDLED
  #define CAN_IRQ_RETVAL  IRQ_RETVAL
#endif /* <=2.5.67 */
#else /*CAN_WITH_RTL*/
  typedef int can_irqreturn_t;
  #define CAN_IRQ_NONE        0
  #define CAN_IRQ_HANDLED     1
  #define CAN_IRQ_RETVAL(x)   ((x) != 0)
#endif /*CAN_WITH_RTL*/

#if (LINUX_VERSION_CODE < KERNEL_VERSION(2,5,33))
   #define can_synchronize_irq(irqnum) synchronize_irq()
#else /* >=2.5.33 */
   #define can_synchronize_irq synchronize_irq
#endif

#if (LINUX_VERSION_CODE < KERNEL_VERSION(2,4,0))
  #define del_timer_sync del_timer
#endif /* <2.4.0 */

#define CAN_ENABLE_KERN_FASYNC

#ifndef CAN_WITH_RTL
/* Standard LINUX kernel */

#define can_spinlock_t             spinlock_t
#define can_spin_irqflags_t        unsigned long
#define can_spin_lock              spin_lock
#define can_spin_unlock            spin_unlock
#define can_spin_lock_irqsave      spin_lock_irqsave
#define can_spin_unlock_irqrestore spin_unlock_irqrestore
#define can_spin_lock_init         spin_lock_init

#if defined(CONFIG_PREEMPT) || (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0))
#define can_preempt_disable        preempt_disable
#define can_preempt_enable         preempt_enable
#else /*CONFIG_PREEMPT*/
#define can_preempt_disable()      do { } while (0)
#define can_preempt_enable()       do { } while (0)
#endif /*CONFIG_PREEMPT*/

#define can_enable_irq             enable_irq
#define can_disable_irq            disable_irq

#define can_printk                 printk

#else /*CAN_WITH_RTL*/

#define can_spinlock_t             rtl_spinlock_t
#define can_spin_irqflags_t        unsigned long
#define can_spin_lock              rtl_spin_lock
#define can_spin_unlock            rtl_spin_unlock
#define can_spin_lock_irqsave      rtl_spin_lock_irqsave
#define can_spin_unlock_irqrestore rtl_spin_unlock_irqrestore
#define can_spin_lock_init         rtl_spin_lock_init

#define can_preempt_disable()      do { } while (0)
#define can_preempt_enable()       do { } while (0)

#define can_enable_irq             rtl_hard_enable_irq
#define can_disable_irq            rtl_hard_disable_irq

#define can_printk                 rtl_printf

#endif /*CAN_WITH_RTL*/

#endif /*_CAN_SYSDEP_H*/
