/* close.c
 * Linux CAN-bus device driver.
 * Written by Arnaud Westenberg email:arnaud@wanadoo.nl
 * Rewritten for new CAN queues by Pavel Pisa - OCERA team member
 * email:pisa@cmp.felk.cvut.cz
 * This software is released under the GPL-License.
 * Version lincan-0.2  9 Jul 2003
 */

#include "../include/can.h"
#include "../include/can_sysdep.h"
#include "../include/main.h"
#include "../include/close.h"
#include "../include/i82527.h"
#include "../include/setup.h"
#include "../include/fasync.h"

#define __NO_VERSION__
#include <linux/module.h>

int can_close(struct inode *inode, struct file *file)
{
	struct canuser_t *canuser = (struct canuser_t*)(file->private_data);
	struct canque_ends_t *qends;
	struct msgobj_t *obj;
	
	if(!canuser || (canuser->magic != CAN_USER_MAGIC)){
		CANMSG("can_close: bad canuser magic\n");
		return -ENODEV;
	}
	
	obj = canuser->msgobj;
	qends = canuser->qends;

    #ifdef CAN_ENABLE_KERN_FASYNC

	can_fasync(-1, file, 0);

    #endif /*CAN_ENABLE_KERN_FASYNC*/

	list_del(&canuser->peers);
	canuser->qends = NULL;
	canqueue_ends_dispose_kern(qends, file->f_flags & O_SYNC);

	kfree(canuser);

	if(atomic_dec_and_test(&obj->obj_used)){
		can_msgobj_clear_fl(obj,OPENED);
		/* FIXME: what about clearing chip HW status, stopping sending messages etc? */
	};
	
    #if (LINUX_VERSION_CODE < KERNEL_VERSION(2,5,50))
	MOD_DEC_USE_COUNT;
    #endif
	return 0;
}
